@@ -70,10 +70,14 @@ class Template(object):
         return self.get_default_value(view.name)
 
     def get_default_value(self, key_name='default'):
-        if self.default is REQUIRED:
-            # Missing required value. This is an error.
+        """Get the default value to return when the value is missing.
+
+        May raise a `NotFoundError` if the value is required.
+        """
+        if not hasattr(self, 'default') or self.default is REQUIRED:
+            # The value is required. A missing value is an error.
             raise exceptions.NotFoundError(u"{} not found".format(key_name))
-        # Missing value, but not required.
+        # The value is not required.
         return self.default
 
     def convert(self, value, view):
@@ -177,7 +181,7 @@ class Sequence(Template):
         """Get a list of items validated against the template.
         """
         out = []
-        for item in view:
+        for item in view.sequence():
             out.append(self.subtemplate.value(item, self))
         return out
 
@@ -185,6 +189,33 @@ class Sequence(Template):
         return 'Sequence({0})'.format(repr(self.subtemplate))
 
 
+class MappingValues(Template):
+    """A template used to validate mappings of similar items,
+    based on a given subtemplate applied to the values.
+
+    All keys in the mapping are considered valid, but values
+    must pass validation by the subtemplate. Similar to the
+    Sequence template but for mappings.
+    """
+    def __init__(self, subtemplate):
+        """Create a template for a mapping with variable keys
+        and item values validated on a given subtemplate.
+        """
+        self.subtemplate = as_template(subtemplate)
+
+    def value(self, view, template=None):
+        """Get a dict with the same keys as the view and the
+        value of each item validated against the subtemplate.
+        """
+        out = {}
+        for key, item in view.items():
+            out[key] = self.subtemplate.value(item, self)
+        return out
+
+    def __repr__(self):
+        return 'MappingValues({0})'.format(repr(self.subtemplate))
+
+
 class String(Template):
     """A string configuration value template.
     """
@@ -431,24 +462,32 @@ class Filename(Template):
     Filenames are returned as absolute, tilde-free paths.
 
     Relative paths are relative to the template's `cwd` argument
-    when it is specified, then the configuration directory (see
-    the `config_dir` method) if they come from a file. Otherwise,
-    they are relative to the current working directory. This helps
-    attain the expected behavior when using command-line options.
+    when it is specified. Otherwise, if the paths come from a file,
+    they will be relative to the configuration directory (see the
+    `config_dir` method) by default or to the base directory of the
+    config file if either the source has `base_for_paths` set to True
+    or the template has `in_source_dir` set to True. Paths from sources
+    without a file are relative to the current working directory. This
+    helps attain the expected behavior when using command-line options.
     """
     def __init__(self, default=REQUIRED, cwd=None, relative_to=None,
-                 in_app_dir=False):
+                 in_app_dir=False, in_source_dir=False):
         """`relative_to` is the name of a sibling value that is
         being validated at the same time.
 
         `in_app_dir` indicates whether the path should be resolved
         inside the application's config directory (even when the setting
         does not come from a file).
+
+        `in_source_dir` indicates whether the path should be resolved
+        relative to the directory containing the source file, if there is
+        one, taking precedence over the application's config directory.
         """
         super(Filename, self).__init__(default)
         self.cwd = cwd
         self.relative_to = relative_to
         self.in_app_dir = in_app_dir
+        self.in_source_dir = in_source_dir
 
     def __repr__(self):
         args = []
@@ -465,6 +504,9 @@ class Filename(Template):
         if self.in_app_dir:
             args.append('in_app_dir=True')
 
+        if self.in_source_dir:
+            args.append('in_source_dir=True')
+
         return 'Filename({0})'.format(', '.join(args))
 
     def resolve_relative_to(self, view, template):
@@ -543,6 +585,11 @@ class Filename(Template):
                     path,
                 )
 
+            elif ((source.filename and self.in_source_dir)
+                  or (source.base_for_paths and not self.in_app_dir)):
+                # relative to the directory the source file is in.
+                path = os.path.join(os.path.dirname(source.filename), path)
+
             elif source.filename or self.in_app_dir:
                 # From defaults: relative to the app's directory.
                 path = os.path.join(view.root().config_dir(), path)
@@ -567,6 +614,50 @@ class Path(Filename):
         return pathlib.Path(value)
 
 
+class Optional(Template):
+    """A template that makes a subtemplate optional.
+
+    If the value is present and not null, it must validate against the
+    subtemplate. However, if the value is null or missing, the template will
+    still validate, returning a default value. If `allow_missing` is False,
+    the template will not allow missing values while still permitting null.
+    """
+
+    def __init__(self, subtemplate, default=None, allow_missing=True):
+        self.subtemplate = as_template(subtemplate)
+        if default is None:
+            # When no default is passed, try to use the subtemplate's
+            # default value as the default for this template
+            try:
+                default = self.subtemplate.get_default_value()
+            except exceptions.NotFoundError:
+                pass
+        self.default = default
+        self.allow_missing = allow_missing
+
+    def value(self, view, template=None):
+        try:
+            value, _ = view.first()
+        except exceptions.NotFoundError:
+            if self.allow_missing:
+                # Value is missing but not required
+                return self.default
+            # Value must be present even though it can be null. Raise an error.
+            raise exceptions.NotFoundError(u'{} not found'.format(view.name))
+
+        if value is None:
+            # None (ie, null) is always a valid value
+            return self.default
+        return self.subtemplate.value(view, self)
+
+    def __repr__(self):
+        return 'Optional({0}, {1}, allow_missing={2})'.format(
+            repr(self.subtemplate),
+            repr(self.default),
+            self.allow_missing,
+        )
+
+
 class TypeTemplate(Template):
     """A simple template that checks that a value is an instance of a
     desired Python type.
